@@ -13,7 +13,6 @@ import statistics
 from fmqlutils import VISTA_DATA_BASE_DIR
 from fmqlutils.cacher.cacherUtils import SCHEMA_LOCN_TEMPL 
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent
-from fmqlreports.webReportUtils import vistasOfVISNByOne
 
 """
 # TODO: move fmqr/directory
@@ -44,6 +43,14 @@ class CommonFileDirectory():
     # - count pointers, literals etc
     # - boolean on pointers to PATIENT etc
     ENTRY_IDX_FIELDCOUNT = 5
+    
+    # BUILD/INSTALL
+    ENTRY_IDX_BUILD_FIRST_DT = 6
+    ENTRY_IDX_BUILD_LAST_DT = 7
+    ENTRY_IDX_BUILD_COUNT = 8
+    ENTRY_IDX_INSTALL_FIRST_DT = 9
+    ENTRY_IDX_INSTALL_LAST_DT = 10
+    ENTRY_IDX_INSTALL_COUNT = 11
 
     def __init__(self, vistaSNOs):
         self.__vistaSNOs = vistaSNOs
@@ -488,12 +495,14 @@ string.isalpha() Returns True if there are only alphabetic characters in the str
         )
         
 # ############################## DRIVER ##########################
-                
+        
+VISN20_VISTAS = {"668": "SPO", "663": "PUG", "648": "POR", "687": "WWW", "531": "BOI", "692": "WCO", "653": "ROS"}
+        
 def main():
 
     assert sys.version_info >= (3, 6)
     
-    cfd = CommonFileDirectory(vistasOfVISNByOne("20"))
+    cfd = CommonFileDirectory(VISN20_VISTAS)
     print(cfd.report())
             
 if __name__ == "__main__":
