@@ -12,8 +12,9 @@ from random import randint
 
 from fmqlutils import VISTA_DATA_BASE_DIR
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportAbsAndPercent
-from fmqlreports.webReportUtils import reduce200, vistasOfVISNByOne
+from fmqlreports.webReportUtils import reduce200
 
+from fileDirectory import VISN20_VISTAS # TMP - move to webReportUtils
 
 """
 TODO FOR PATIENT/USER RE-ALIGN + PLOT EASY -- make the directories align and
@@ -396,21 +397,11 @@ class CommonUserDirectory():
         sdeid = cud.subsetNDEID(allGIENs, IDMaker(), NameMaker())
     
     To see: pass in nm.report()
-    
-    preGeneratedNames = {"SNO:GIEN": FULL NAME - FIRST MID LAST} and then all
-    other entries with that user ("SNO1:GIEN1" ...) will share this same name
-    setup
     """
-    def subsetNDEID(self, userGIENs, idMaker, nameMaker, preGeneratedNames={}):
+    def subsetNDEID(self, userGIENs, idMaker, nameMaker):
         nusers = []
         nentries = {}
         snos = set()
-        if len(preGeneratedNames):
-            fnsExcluded = set(
-               "{} {}".format(nm.split(" ")[0], nm.split(" ")[-1])
-                for nm in preGeneratedNames.values()
-            )
-            nameMaker.excludeNames(fnsExcluded)            
         for i, user in enumerate(self.__udirInfo["users"]):
             if len(userGIENs) and sum(1 for gien in user if gien in userGIENs) == 0:
                 continue
@@ -424,15 +415,7 @@ class CommonUserDirectory():
                 print("** sexless user: {} - picking  {} 50/50".format(self.__udirInfo["entries"][user[0]][1], sex))
             else:
                 sex = "male" if list(sexes)[0] == "M" else "female"
-            nms = []
-            if len(preGeneratedNames): # insist on full: first middle last
-                for userGIEN in user:
-                    if userGIEN in preGeneratedNames:
-                        nms = preGeneratedNames[userGIEN].split(" ")
-                        break
-            if len(nms) == 0:
-                nms = nameMaker.getNames(gender=sex, originalName=self.__udirInfo["entries"][user[0]][1])
-            name, nameInitials, iname, inameInitials, nwnmEnd = nameMaker.calculateUserNameVariations(nms)
+            name, nameInitials, iname, inameInitials, nwnmEnd = nameMaker.getUserNames(gender=sex, originalName=self.__udirInfo["entries"][user[0]][1])
             origNWNM = ""
             for gien in user:
                 snos.add(gien.split(":")[0])
@@ -777,8 +760,9 @@ def main():
 
     assert sys.version_info >= (3, 6)
     
-    cud = CommonUserDirectory(vistasOfVISNByOne("20"))
+    cud = CommonUserDirectory(VISN20_VISTAS)
     print(cud.report())
+    json.dump(cud.entryDatesBySNO(), open("Meta/Common/userDatesBySNO.json", "w"), indent=4)
             
 if __name__ == "__main__":
     main()  
