@@ -17,14 +17,15 @@ import pandas as pd
 from fmqlutils import VISTA_DATA_BASE_DIR
 from fmqlutils.cacher.cacherUtils import SCHEMA_LOCN_TEMPL 
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent
-from fmqlreports.webReportUtils import SITE_DIR_TEMPL, TOP_MD_TEMPL, keyStats, roundFloat, vistasOfVISNByOne
+from fmqlreports.webReportUtils import SITE_DIR_TEMPL, TOP_MD_TEMPL, keyStats, roundFloat
 
-from fmqlreports.webReportUtils import muPlotRef, makePlots 
+from ifc import muPlotRef, makePlots # TMP (go central to visualizeUtils)
+from webReportFileDirectory import VISN20_VISTAS # TMP (go central to webReportUtils)
 from userDirectory import CommonUserDirectory
         
 """
 QUICK TODO: 
-- reorder the render of user totals in time series (last graph) so biggest appears first ie/ key has PUG, POR ... colNameById needs to be ordered
+- reorder the render of user totals in time series (last graph) so biggest appears first ie/ key has PUG, POR ... vistaNamesPlus needs to be ordered
 
 Goal for directories in general - user ala patient with one, move id checks there but
 first need to see what's there now. Federated Directory => mismatch possible, dup,
@@ -172,8 +173,8 @@ def makeTimeSeriesSection(cd, vistaNames, imageDir="Images"):
     
     dailiesDFCSV = csvDatesBySNO(cd)
         
-    colNameById = vistaNames.copy()
-    colNameById["USER"] = "New" # new users
+    vistaNamesPlus = vistaNames.copy()
+    vistaNamesPlus["USER"] = "New" # new users
     
     plotData["tsDailies"] = {
         "title": "2020/01-{} User Addition".format(re.sub(r'\-', '/', end)),
@@ -181,7 +182,7 @@ def makeTimeSeriesSection(cd, vistaNames, imageDir="Images"):
         "plotMethod": "plotDailies",
         "dfcsv": dailiesDFCSV,
         "kargs": {
-            "colNameById": colNameById,
+            "vistaNamesPlus": vistaNamesPlus,
             "start": "2020-01",
             "end": end
         }
@@ -201,12 +202,12 @@ def makeTimeSeriesSection(cd, vistaNames, imageDir="Images"):
         "plotMethod": "plot365RollingMean",
         "dfcsv": dailiesDFCSV,
         "kargs": {
-            "colNameById": colNameById,
+            "vistaNamesPlus": vistaNamesPlus,
             "start": "1987-01", 
             "end": end
         }
     }
-    blurb = "Here are the yearly trends using a __\"365 day rolling mean\"__ starting when the directory first gets data to the latest additions. In general, additions follow system size but there's an exception in 2019 where four of the seven have a large spike in additions and Portland exceeds Puget's total. This is one of four times where the rate of addition changes - the others are 2002, 2006 and 2009. _All need examination_ - they must reflect the addition of or changes to VistA applications and use."
+    blurb = "Here are the yearly trends using a __\"365 day rolling mean\"__ starting when the directory first gets data to the latest additions. In general, additions follow system size (Puget, Portland, Spokane, Walla Walla) but there's an exception in 2019 where four of the seven have a large spike in additions and Portland exceeds Puget's total. This is one of four times where the rate of addition changes - the others are 2002, 2006 and 2009. _All need examination_ - they must reflect the addition of or changes to VistA applications and use."
     mu += "{}\n\n{}\n\n".format(
         blurb,
         muPlotRef(
@@ -221,12 +222,12 @@ def makeTimeSeriesSection(cd, vistaNames, imageDir="Images"):
         "plotMethod": "plotMonthlyResamples",
         "dfcsv": dailiesDFCSV,
         "kargs": {
-            "colNameById": colNameById,
+            "vistaNamesPlus": vistaNamesPlus,
             "start": '2018-01', 
             "end": end
         }
     }
-    blurb = "When the data is __resampled by month__, it shows three subpeaks for the 2019 leap in additions. The reason for Portland's leap above Puget also shows more clearly - though Puget has one large spike in April 2019, Portland has two, one in July and one in September. Note that the lack of any spike mirrors the flat line for smaller systems during this period in the yearly trends above.\n\nThis graph also shows (\"New\") the running total of new users added. As seen above, users added to VistAs sometimes already pre-exist in other VistAs. While the new user line shadows Puget's peak in April and Portland's in September, it is markedly lower than the spikes for Portland and Spokane in July - this indicates that much of those spikes are existing (Puget?) users being added to these systems. _This needs examination_."
+    blurb = "When the data is __resampled by month__, it shows three subpeaks for the 2019 leap in additions. The reason for Portland's leap above Puget also shows more clearly - though Puget has one large spike in April 2019, Portland has two, one in July and one in September. Note that the lack of any spike mirrors the flat line for smaller systems during this period in the yearly trends above.\n\nThis graph also shows - in purple - the running total of new users added. As seen above, users added to VistAs sometimes already pre-exist in other VistAs. While the new user line shadows Puget's peak in April and Portland's in September, it is markedly lower than the spikes for Portland and Spokane in July - this indicates that much of those spikes are existing (Puget?) users being added to these systems. _This needs examination_."
     mu += "{}\n\n{}\n\n".format(
         blurb,
         muPlotRef(
@@ -241,7 +242,7 @@ def makeTimeSeriesSection(cd, vistaNames, imageDir="Images"):
         "plotMethod": "plotTotalsPerYear",
         "dfcsv": dailiesDFCSV,
         "kargs": {
-            "colNameById": colNameById,
+            "vistaNamesPlus": vistaNamesPlus,
             "start": "2015", 
             "end": end
         }
@@ -376,9 +377,8 @@ MORE:
 - https://levelup.gitconnected.com/pandas-dataframe-python-10-useful-tricks-b4beae91df3d ... filtering, combining, renaming in frames ie/ all frame operations to try
 - https://medium.com/@rrfd/cleaning-and-prepping-data-with-python-for-data-science-best-practices-and-helpful-packages-af1edfbe2a3 <---- includes 'fillna'
 """
-def csvDatesBySNO(cd=None, datesBySNO=None):
-        if datesBySNO == None:
-            datesBySNO = cd.entryDatesBySNO()
+def csvDatesBySNO(cd):
+        datesBySNO = cd.entryDatesBySNO()
         snosOrdered = sorted(datesBySNO, key=lambda x: len(datesBySNO[x]), reverse=True)
         byDateBySNO = defaultdict(Counter)
         for sno in datesBySNO:
@@ -407,10 +407,8 @@ def main():
             return
         raise Exception("Only argument allowed is 'PLOT'")
 
-    visn20VistAs = vistasOfVISNByOne("20")
-
     dirName = "User"
-    cud = CommonUserDirectory(visn20VistAs)
+    cud = CommonUserDirectory(VISN20_VISTAS)
 
     plotData = {}
     mu = f"""---
@@ -431,11 +429,11 @@ Note: Anchorage is the missing VISN 20 system. Its information will be added onc
         cud.entryTotal()
     )
     
-    ttlPlotData, ttlMU = makeTotalsSection(cud, visn20VistAs)
+    ttlPlotData, ttlMU = makeTotalsSection(cud, VISN20_VISTAS)
     mu += ttlMU
     plotData.update(ttlPlotData)
     
-    tsPlotData, tsMU = makeTimeSeriesSection(cud, visn20VistAs)
+    tsPlotData, tsMU = makeTimeSeriesSection(cud, VISN20_VISTAS)
     mu += tsMU
     plotData.update(tsPlotData)
         
