@@ -774,6 +774,10 @@ def gatherEncounterDataByDay(stationNo, onAndAfterDay, upToDay):
                 ))
             if i % 50000 == 0:
                 print("\tAnother 50K to {:,}, {:,} taken, {:,} credit stop visit's skipped, last resource day {} - 50K resource create date {}".format(i, cntEncountersInDateRange, cntQualCreditStopVisitsSkipped, resourceDay, resource[startProp]["value"] if startProp in resource else "?"))
+            # issue in one of 648
+            if "service_category" not in resource:
+                print("** Skipping Visit as no setting for 'service_category'")
+                continue
             # Keeps it in the window of onAndAfterDay and upToDay
             resourceDay = getResourceDay(resource, createProp, onAndAfterDay, upToDay)
             if resourceDay == "":
@@ -996,6 +1000,7 @@ def indexEncounterDataByDateLocationPatient(stationNo, visitsByDay, oldApptsByDa
                 noHospitalLocationVisitsByPackage[res["package"]] += 1
                 continue
             locnLabel = toLocnLabel(res["hospital_location"])
+            # TODO: BUT perhaps exclude res's w/o it (saw in 648)
             if res["service_category"] == "E:EVENT (HISTORICAL)": # some have all keys props so exclude em all
                 if "package" in res and re.match(r'SCHEDULING', res["package"]):
                     raise Exception("Dont expect EVENT HISTORICAL service_category with package SCHEDULING")
@@ -1032,14 +1037,16 @@ def categorizeVisitAppts(info, noneApptOnlyDetails=None, matchedDetails=None, ma
         # Strict for apptN - looser (need apptL) for apptL (TODO: tighten)
         if len(apptProps) > 1:
             if apptPrefix == "apptN" or apptPrefix not in info:  
-                raise Exception("Ambiguous multiple {} matches".format(apptPrefix))
-            apptProp = apptPrefix
+                # USED TO BE AN EXCEPTION BUT ISSUE FOR POR 663 TODO
+                print("*** Ambiguous multiple {} matches".format(apptPrefix, json.dumps(apptProps)))
+            # apptProp = apptPrefix
+            apptProp = apptPrefix if apptPrefix in apptProps else apptProps[0]
         else:
             apptProp = apptProps[0]
         # Strict for apptN - looser - pick bigger for apptL (TODO: tighten)
         if len(info[apptProp]) > 1:
-            if apptPrefix == "apptN": # Strict
-                raise Exception("Ambiguous multiple {}s at same time".format(apptPrefix))
+            if apptPrefix == "apptN": # Warning for apptN (weakened exp for 663)
+                print("** Ambiguous multiple {}s at same time".format(apptPrefix))
             ressBySize = sorted(info[apptProp], key=lambda x: len(x), reverse=True)
             appt = ressBySize[0] # ie/ with checkout etc     
         else:
