@@ -732,6 +732,83 @@ def mdFilesOfDomain(stationNo, domainName, bottomFileNo, beyondFileNo, usedFiles
         
     return mu
     
+# ##################### Plot Creation #########################
+        
+"""
+Plot creation is separate --- may move into Visualize so common
+
+ex/ plotDataFile = "plotDataIFC.json" ie/ plotDataFileSuffix="IFC"
+
+TODO:
+- ifcPlTravelVISNSankey (from Travel PlacersVISNSankey) -- .md change too
+- all consult services categ ... -> ifcAllConsultsCategory 
+- remove 0 ie/ NO OTHER!
+- map SEA -> PUG, MAN -> SPO
+
+"""
+
+def makePlots(stationNo, plotDataFileSuffix):   
+
+    import io
+    import pandas as pd
+
+    from visualize import Visualize
+    imageDir = f'{VISTA_DATA_BASE_DIR}{stationNo}/ReportSite/Images
+    if not os.path.isdir(imageDir):
+        os.mkdir(imageDir)
+    viz = Visualize(imageDir)
+    plotData =  json.load(open(f'{VISTA_DATA_BASE_DIR}{stationNo}/TmpWorking/plotData{plotDataFileSuffix}.json'))
+    for plotDataName in plotData:
+        plotDataInfo = plotData[plotDataName]
+        print(f'Remaking {plotDataInfo["title"]}')
+        method = getattr(viz, plotDataInfo["plotMethod"])
+        if "rows" in plotDataInfo:
+            arg = viz.makeDF(
+                data=plotDataInfo["data"],
+                columns=plotDataInfo["columns"],
+                rows=plotDataInfo["rows"]
+            )
+        elif "specs" in plotDataInfo:
+            arg = plotDataInfo["specs"]
+        elif "dfcsv" in plotDataInfo:
+            dfcsv = plotDataInfo["dfcsv"]
+            df = pd.read_csv(
+                io.StringIO("\n".join(dfcsv)),
+                index_col=0, 
+                parse_dates=True
+            )
+            arg = df
+        else:
+            raise Exception("Expect plot data for DF or as 'specs'")
+        kargs = plotDataInfo.get("kargs", {})
+        plotRef = method(arg, plotDataInfo["title"], plotDataInfo["plotName"], **kargs)
+        print(f'\texpect reference to {plotRef} in report')
+        
+def muPlotRef(plotInfo, flushToDirectory):
+    """
+    # Want separate from visualize for now BUT possible break
+    # as separate from Visualize naming impl
+    
+    ex/ ![ifcTravelVISNSankey](Images/ifcTravelVISNSankeyBGW.svg)
+    """
+    def svgPlotFile(plotName, plotMethodName, plotKArgs, flushToDirectory):
+        """
+        So don't have to make the plot but can know its name as making md
+        Compatible with Visualize.__flushPlot's naming of SVG's
+        """
+        coreMethodName = re.sub("plot", "", plotMethodName)
+        coreMethodNameU = coreMethodName[0].upper() + coreMethodName[1:]
+        plotFilePrefix = f'{plotName}{coreMethodNameU}'
+        if plotKArgs:
+            if "start" in plotKArgs:
+                plotFilePrefix += plotKArgs["start"]
+            if "end" in plotKArgs:
+                plotFilePrefix += "-{}".format(plotKArgs["end"])
+        plotFile = f'{flushToDirectory}/{plotFilePrefix}BGW.svg'
+        return plotFile, plotFilePrefix
+    plotFile, plotFilePrefix = svgPlotFile(plotInfo["plotName"], plotInfo["plotMethod"], plotInfo["kargs"] if "kargs" in plotInfo else None, flushToDirectory)
+    return "![{}]({})".format(plotFilePrefix, plotFile)
+    
 # ######################### Simple Type Utils ########################
 
 class PropertyCounter:
